package org.opendof.tools.repository.interfaces.servlet.menu.sharing;

import org.apache.wicket.markup.html.basic.Label;
import org.apache.wicket.markup.html.form.Button;
import org.apache.wicket.markup.html.form.CheckBox;
import org.apache.wicket.markup.html.form.Form;
import org.apache.wicket.markup.html.form.TextField;
import org.apache.wicket.markup.html.list.ListItem;
import org.apache.wicket.model.Model;
import org.apache.wicket.model.PropertyModel;
import org.opendof.tools.repository.interfaces.core.AuthenticatedUser;
import org.opendof.tools.repository.interfaces.core.CoreController;
import org.opendof.tools.repository.interfaces.da.InterfaceData;
import org.opendof.tools.repository.interfaces.da.SubmitterPermissions;
import org.opendof.tools.repository.interfaces.servlet.menu.InterfacePanel;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import org.apache.wicket.markup.html.list.ListView;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.Objects;

/**
 * Created by dethington on 7/17/2017.
 */
public class ManageSharingPanel extends InterfacePanel {

    private final InterfaceData interfaceData;
    private final AuthenticatedUser authUser;
    private static final Logger log = LoggerFactory.getLogger(ManageSharingPanel.class);
    private ListView<PermissionsWrapper> listView;


    public ManageSharingPanel(String id, InterfaceData interfaceData) {
        super(id);
        this.interfaceData = interfaceData;
        authUser = getClientInformation().isAuthenticated() ? getAuthUser() : null;
        loadSubmitterPermissionList();
        add(new AddSubmitterForm("newsubmitter"));
        add(listView);
    }

    private void addNewRow(String email, SubmitterPermissions.PermissionLevel permission) {
        try {

            getCore().setPermissions(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.version,
                    email, permission.getValue());
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void loadSubmitterPermissionList(){
        List<PermissionsWrapper> permissionsList = new ArrayList<>();
        if(authUser != null){
            try {
                List<SubmitterPermissions> submitterPermissionsList = getCore().getPermissions(authUser, interfaceData.getRepoType(), interfaceData.iid, interfaceData.version);
                for(SubmitterPermissions permissions : submitterPermissionsList){
                    permissionsList.add(new PermissionsWrapper(permissions));
                }
            } catch(Exception e){
                log.debug("Failed to get permissions.  Exception: {}", e.getMessage());
            }
        }

        listView = new ListView<PermissionsWrapper>("listView", permissionsList){
            private static final long serialVersionUID = 5374952604176542286L;

            @Override
            protected void populateItem(ListItem<PermissionsWrapper> item){
                PermissionsWrapper submitterPermission = item.getModelObject();
                item.add(new Label("email", submitterPermission.getEmail()));
                item.add(new CheckBox("cb", new BooleanPropertyModel(item)));
            }
        };
    }

    private static final class BooleanPropertyModel extends PropertyModel<Boolean> {
        private static final long serialVersionUID = 2794620818528581500L;

        private BooleanPropertyModel(ListItem<PermissionsWrapper> item) {
            super(item, "selected");

        }

        private boolean wantOnSelectionChangedNotifications(){
            return true;
        }
    }

    private static class PermissionsWrapper implements Serializable {
        private static final long serialVersionUID = -6072944005630711339L;

        private final String email;
        private Boolean selected = Boolean.FALSE;

        PermissionsWrapper(SubmitterPermissions permissions){
            email = permissions.submitter.email;
            selected = Objects.equals(permissions.permission, SubmitterPermissions.PermissionLevel.WRITE);
        }

        public Boolean getSelected() {
            return selected;
        }

        public void setSelected(Boolean selected) {
            this.selected = selected;
        }

        public String getEmail() {
            return email;
        }
    }

    private class AddSubmitterForm extends Form {
        private TextField emailField;
        private CheckBox canWrite;
        private Button submit;

        AddSubmitterForm(String id){
            super(id);

            emailField = new TextField("emailField", Model.of(""));
            canWrite = new CheckBox("canWrite", Model.of(Boolean.FALSE) );
            submit = new Button("submit");
            add(emailField);
            add(canWrite);
            add(submit);
        }

        public final void onSubmit(){
            String email = emailField.getDefaultModelObjectAsString();
            Boolean hasWrite = canWrite.getModelObject();
            SubmitterPermissions.PermissionLevel permissionLevel = SubmitterPermissions.PermissionLevel.READ;
            if(hasWrite)
                permissionLevel = SubmitterPermissions.PermissionLevel.WRITE;
            addNewRow(email, permissionLevel);
        }
    }
}
